@@ -42,7 +42,7 @@ namespace acommon {
     virtual ~Decode() {}
   };
   struct Encode : public ConvBase {
-    // null characters should be tretead like any other character
+    // null characters should be treated like any other character
     // by the encoder.
     virtual PosibErr<void> init(ParmString, const Config &) {return no_err;}
     virtual void encode(const FilterChar * in, const FilterChar * stop, 
@@ -55,10 +55,10 @@ namespace acommon {
     static PosibErr<Encode *> get_new(const String &, const Config *);
     virtual ~Encode() {}
   };
-  struct DirectConv  { // convert directly from in_code to out_code
-    // should not take owenership of decode and encode 
+  struct DirectConv { // convert directly from in_code to out_code.
+    // should not take ownership of decode and encode.
     // decode and encode guaranteed to stick around for the life
-    // of the object
+    // of the object.
     virtual PosibErr<void> init(const Decode *, const Encode *, 
 				const Config &) {return no_err;}
     virtual void convert(const char * in, int size, 
