@@ -9,6 +9,7 @@
 
 #include <stddef.h>
 #include <limits.h>
+#include "string.hpp"
 #include "mutable_string.hpp"
 
 namespace acommon {
@@ -33,6 +34,12 @@ namespace acommon {
     char * e = unescape(d.str, d.str);
     d.size = e - d.str;
   }
+  static inline void unescape(String & d) {
+    d.ensure_null_end();
+    char * e = unescape(d.data(), d.data());
+    d.resize(e - d.data());
+  }
+  
 
   // if limit is not given than dest should have enough space for 
   // twice the number of characters of src
@@ -41,10 +48,10 @@ namespace acommon {
 
   void to_lower(char *);
 
-  // extract the first whitespace delemited word from d.value and put
+  // extract the first whitespace delimited word from d.value and put
   // it in d.key.  d.value is expected not to have any leading
-  // whitespace. The string is modifed in place so that d.key will be
-  // null terminated.  d.value is modifed to point to any additional
+  // whitespace. The string is modified in place so that d.key will be
+  // null terminated.  d.value is modified to point to any additional
   // data after key with any leading white space removed.  Returns
   // false when there are no more words to extract
   bool split(DataPair & d);
