@@ -388,28 +388,23 @@ namespace acommon {
     to_add->code.assign(name, p0-name);
     to_add->c_struct.code = to_add->code.c_str();
 
-    // check if the code is in a valid form and normilize entry.  
+    // check if the code is in a valid form and normalize entry.  
     // If its not in a valid form than ignore this entry
 
     if (to_add->code.size() >= 2 
 	&& asc_isalpha(to_add->code[0]) && asc_isalpha(to_add->code[1])) 
     {
-      to_add->name[0] = asc_tolower(to_add->name[0]);
-      to_add->name[1] = asc_tolower(to_add->name[1]);
-      to_add->code[0] = asc_tolower(to_add->code[0]);
-      to_add->code[1] = asc_tolower(to_add->code[1]);
-      if (to_add->code.size() == 2); // do nothing
-      else if (to_add->code.size() == 5 && to_add->code[2] == '_' 
-	       && asc_isalpha(to_add->code[3]) 
-	       && asc_isalpha(to_add->code[4])) {
-	to_add->name[3] = asc_toupper(to_add->name[3]);
-	to_add->name[4] = asc_toupper(to_add->name[4]);
-	to_add->code[3] = asc_toupper(to_add->code[3]);
-	to_add->code[4] = asc_toupper(to_add->code[4]);
-      } else
-	return no_err;
-    } else
+      int s = strcspn(to_add->code.str(), "_");
+      if (s > 3) return no_err;
+      int i = 0;
+      for (; i != s; ++i)
+        to_add->name[i] = to_add->code[i] = asc_tolower(to_add->code[i]);
+      i++;
+      for (; i < to_add->code.size(); ++i)
+        to_add->name[i] = to_add->code[i] = asc_toupper(to_add->code[i]);
+    } else {
       return no_err;
+    }
     
     // Need to do it here as module is about to get a value
     // if it is null
@@ -673,7 +668,7 @@ namespace acommon {
 
   /////////////////////////////////////////////////////////////////
   //
-  // user visable functions and enumeration impl
+  // user visible functions and enumeration impl
   //
 
   //
