@@ -24,6 +24,7 @@ namespace acommon {
   class StringList;
   struct StringListImpl;
   class FStream;
+  class StringMap;
 
   struct ModuleInfo {
     const char * name;
@@ -39,7 +40,7 @@ namespace acommon {
   struct DictInfo {
     const char * name;
     const char * code;
-    const char * jargon;
+    const char * variety;
     int size;
     const char * size_str;
     const ModuleInfo * module;
@@ -94,6 +95,8 @@ namespace acommon {
 
   const DictInfoList * get_dict_info_list(Config *);
 
+  const StringMap * get_dict_aliases(Config *);
+
   class ModuleInfoEnumeration {
   public:
     typedef const ModuleInfo * Value;
@@ -137,6 +140,8 @@ namespace acommon {
   };
 
 
+
+
 }
 
 #endif /* ASPELL_INFO__HPP */
