@@ -4,12 +4,13 @@
 // license along with this library if you did not you can find
 // it at http://www.gnu.org/.
 
-#include "settings.h"
+#define COMPILE_IN_FILTER // URL filter is always static
+
 #include "indiv_filter.hpp"
 #include "key_info.hpp"
+#include "loadable-filter-API.hpp"
 
-namespace {
-  using namespace acommon;
+namespace acommon {
 
   class UrlFilter : public IndividualFilter {
   public:
@@ -57,10 +58,6 @@ namespace {
       }
     }
   }
-}
 
-C_EXPORT 
-IndividualFilter * new_aspell_url_filter() {
-  return new UrlFilter;                                
+  ACTIVATE_FILTER(acommon,UrlFilter,url)
 }
-
