@@ -66,7 +66,7 @@ static char EMPTY[1] = {0};
 
 //////////////////////////////////////////////////////////////////////
 //
-//*Entry struct definations
+// Entry struct definations
 //
 
 struct Conds
@@ -79,11 +79,11 @@ struct Conds
 
 struct AffEntry
 {
-  char *         appnd;
-  char *         strip;
-  unsigned int   appndl;
-  unsigned int   stripl;
-  int            xpflg;
+  const char *   appnd;
+  const char *   strip;
+  byte           appndl;
+  byte           stripl;
+  byte           xpflg;
   char           achar;
   const Conds *  conds;
   //unsigned int numconds;
@@ -94,15 +94,14 @@ struct AffEntry
   
 struct PfxEntry : public AffEntry
 {
-  AffixMgr * pmyMgr;
-  
   PfxEntry * next;
   PfxEntry * next_eq;
   PfxEntry * next_ne;
   PfxEntry * flag_next;
-  PfxEntry(AffixMgr * pmgr) : pmyMgr(pmgr) {}
+  PfxEntry() {}
 
-  bool check(const LookupInfo &, ParmString, CheckInfo &, GuessInfo *) const;
+  bool check(const LookupInfo &, const AffixMgr * pmyMgr,
+             ParmString, CheckInfo &, GuessInfo *) const;
 
   inline bool          allow_cross() const { return ((xpflg & XPRODUCT) != 0); }
   inline byte flag() const { return achar;  }
@@ -115,15 +114,14 @@ struct PfxEntry : public AffEntry
 
 struct SfxEntry : public AffEntry
 {
-  AffixMgr*    pmyMgr;
-  char *       rappnd; // this is set in AffixMgr::build_sfxlist
+  const char * rappnd; // this is set in AffixMgr::build_sfxlist
   
   SfxEntry *   next;
   SfxEntry *   next_eq;
   SfxEntry *   next_ne;
   SfxEntry *   flag_next;
 
-  SfxEntry(AffixMgr* pmgr) : pmyMgr(pmgr) {}
+  SfxEntry() {}
 
   bool check(const LookupInfo &, ParmString, CheckInfo &, GuessInfo *,
              int optflags, AffEntry * ppfx);
@@ -167,6 +165,28 @@ struct AffixLess
   bool operator() (T * x, T * y) const {return strcmp(x->key(),y->key()) < 0;}
 };
 
+// struct StringLookup {
+//   struct Parms {
+//     typedef const char * Value;
+//     typedef const char * Key;
+//     static const bool is_multi = false;
+//     hash<const char *> hfun;
+//     size_t hash(const char * s) {return hfun(s);}
+//     bool equal(const char * x, const char * y) {return strcmp(x,y) == 0;}
+//     const char * key(const char * c) {return c;}
+//   };
+//   typedef HashTable<Parms> Lookup;
+//   Lookup lookup;
+//   ObjStack * data_buf;
+//   StringLookup(ObjStack * b) : data_buf(b) {}
+//   const char * dup(const char * orig) {
+//     pair<Lookup::iterator, bool> res = lookup.insert(orig);
+//     if (res.second) *res.first = data_buf->dup(orig);
+//     return *res.first;
+//     //return data_buf->dup(orig);
+//   }
+// };
+
 struct CondsLookupParms {
   typedef const Conds * Value;
   typedef const char * Key;
@@ -179,6 +199,40 @@ struct CondsLookupParms {
 
 typedef HashTable<CondsLookupParms> CondsLookup;
 
+// normalizes and checks the cond_str
+// returns the lenth of the new string or -1 if invalid
+static int normalize_cond_str(char * str)
+{
+  char * s = str;
+  char * d = str;
+  while (*s) {
+    if (*s != '[') {
+      *d++ = *s++;
+    } else if (s[1] == '\0' || s[1] == ']') {
+      return -1;
+    } else if (s[2] == ']') {
+      *d++ = s[1];
+      s += 3;
+    } else {
+      *d++ = *s++;
+      if (*s == '^') *d++ = *s++;
+      while (*s != ']') {
+        if (*s == '\0' || *s == '[') return -1;
+        char * min = s;
+        for (char * i = s + 1; *i != ']'; ++i) {
+          if ((byte)*i < (byte)*min) min = i;}
+        char c = *s;
+        *d++ = *min;
+        *min = c;
+        ++s;
+      }
+      *d++ = *s++;
+    }
+  }
+  *d = '\0';
+  return d - str;
+}
+
 static void encodeit(CondsLookup &, ObjStack &, 
                      AffEntry * ptr, char * cs);
 
@@ -231,6 +285,8 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
   // read in each line ignoring any that do not
   // start with a known line type indicator
 
+  char prev_aff = '\0';
+
   while (getdata_pair(afflst,dp,buf)) {
     char affix_type = ' ';
 
@@ -239,11 +295,8 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
     if (dp.key == "SET") {
       String buf;
       encoding = data_buf.dup(fix_encoding_str(dp.value, buf));
-      char msg[96];
-      snprintf(msg, 96, _("Expected the file to be in \"%s\" not \"%s\"."),
-               lang->data_encoding(), encoding);
       if (strcmp(encoding, lang->data_encoding()) != 0)
-        return make_err(bad_file_format, affix_file, msg);
+        return make_err(incorrect_encoding, affix_file, lang->data_encoding(), encoding);
     }
 
     /* parse in the flag used by the controlled compound words */
@@ -274,16 +327,22 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
       split(dp);
       if (dp.key.empty()) goto error;
       // key is affix char
-      achar = iconv(dp.key)[0];
+      const char * astr = iconv(dp.key);
+      if (astr[0] == '\0' || astr[1] != '\0') goto error;
+      achar = astr[0];
+      if (achar == prev_aff) goto error_count;
+      prev_aff = achar;
 
       split(dp);
-      if (dp.key.empty()) goto error;
+      if (dp.key.size != 1 || 
+          !(dp.key[0] == 'Y' || dp.key[0] == 'N')) goto error;
       // key is cross product indicator 
       if (dp.key[0] == 'Y') xpflg = XPRODUCT;
     
       split(dp);
       if (dp.key.empty()) goto error;
       // key is number of affentries
+      
       numents = atoi(dp.key); 
   
       for (int j = 0; j < numents; j++) {
@@ -291,10 +350,10 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
 
         if (affix_type == 'P') {
           nptr = (AffEntry *) data_buf.alloc_bottom(sizeof(PfxEntry));
-          new (nptr) PfxEntry(this);
+          new (nptr) PfxEntry;
         } else {
           nptr = (AffEntry *) data_buf.alloc_bottom(sizeof(SfxEntry));
-          new (nptr) SfxEntry(this);
+          new (nptr) SfxEntry;
         }
 
         nptr->xpflg = xpflg;
@@ -302,12 +361,7 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
         split(dp);
         if (dp.key.empty()) goto error;
         // key is affix charter
-        if (iconv(dp.key)[0] != achar) {
-          char msg[64];
-          snprintf(msg, 64, _("affix '%s' is corrupt, possible incorrect count"), 
-                   MsgConv(lang)(achar));
-          return make_err(bad_file_format, affix_file, msg);
-        }
+        if (iconv(dp.key)[0] != achar) goto error_count;
         nptr->achar = achar;
  
         split(dp);
@@ -320,7 +374,7 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
           nptr->strip = data_buf.dup(s0);
           nptr->stripl = s0.size();
         } else {
-          nptr->strip= data_buf.dup("");
+          nptr->strip  = "";
           nptr->stripl = 0;
         }
     
@@ -328,17 +382,31 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
         if (dp.key.empty()) goto error;
         // key is affix string or 0 for null
         if (dp.key != "0") {
-          nptr->appnd = data_buf.dup(iconv(dp.key));
+          nptr->appnd  = data_buf.dup(iconv(dp.key));
           nptr->appndl = strlen(nptr->appnd);
         } else {
-          nptr->appnd  = data_buf.dup("");
+          nptr->appnd  = "";
           nptr->appndl = 0;
         }
     
         split(dp);
         if (dp.key.empty()) goto error;
         // key is the conditions descriptions
-        encodeit(conds_lookup, data_buf, nptr,iconv(dp.key));
+        char * cond = iconv(dp.key);
+        int cond_len = normalize_cond_str(cond);
+        if (cond_len < 0)
+          return (make_err(invalid_cond, MsgConv(lang)(cond))
+                  .with_file(affix_file, dp.line_num));
+        if (nptr->stripl != 0) {
+          char * cc = cond;
+          if (affix_type == 'S') cc += cond_len - nptr->stripl;
+          if (cond_len < nptr->stripl || 
+              memcmp(cc, nptr->strip, nptr->stripl) != 0)
+            return (make_err(invalid_cond_strip, 
+                             MsgConv(lang)(cond), MsgConv(lang)(nptr->strip))
+                    .with_file(affix_file, dp.line_num));
+        }
+        encodeit(conds_lookup, data_buf, nptr, cond);
     
         // now create SfxEntry or PfxEntry objects and use links to
         // build an ordered (sorted by affix string) list
@@ -350,9 +418,10 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
     }
     continue;
   error:
-    char msg[32];
-    snprintf(msg, 32, _("Affix '%s' is corrupt"), MsgConv(lang)(achar));
-    return make_err(other_error, msg).with_file(affix_file, dp.line_num);
+    return make_err(corrupt_affix, MsgConv(lang)(achar)).with_file(affix_file, dp.line_num);
+  error_count:
+    return make_err(corrupt_affix, MsgConv(lang)(achar), 
+                    _("Possibly incorrect count.")).with_file(affix_file, dp.line_num);
   }
   afflst.close();
 
@@ -387,6 +456,8 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
   process_pfx_order();
   process_sfx_order();
 
+  //CERR.printf("%u\n", data_buf.calc_size()/1024);
+
   return no_err;
 
 }
@@ -419,7 +490,6 @@ PosibErr<void> AffixMgr::build_pfxlist(PfxEntry* pfxptr)
   return no_err;
 }
 
-
 // we want to be able to quickly access suffix information
 // both by suffix flag, and sorted by the reverse of the
 // suffix string itself; so we need to set up two indexes
@@ -428,12 +498,14 @@ PosibErr<void> AffixMgr::build_sfxlist(SfxEntry* sfxptr)
 {
   SfxEntry * ptr;
   SfxEntry * ep = sfxptr;
-  sfxptr->rappnd = (char *)data_buf.alloc(sfxptr->appndl + 1);
+  char * tmp = (char *)data_buf.alloc(sfxptr->appndl + 1);
+  sfxptr->rappnd = tmp;
+
   // reverse the string
-  sfxptr->rappnd[sfxptr->appndl] = 0;
-  for (char * dest = sfxptr->rappnd + sfxptr->appndl - 1, * src = sfxptr->appnd;
-       dest >= sfxptr->rappnd;
-       --dest, ++src)
+  char * dest = tmp + sfxptr->appndl;
+  *dest-- = 0;
+  const char * src = sfxptr->appnd;
+  for (; dest >= tmp; --dest, ++src)
     *dest = *src;
 
   /* get the right starting point */
@@ -561,35 +633,6 @@ PosibErr<void> AffixMgr::process_sfx_order()
   return no_err;
 }
 
-// assumes the cond string is valid
-static void normalize_cond_str(char * str)
-{
-  char * s = str;
-  char * d = str;
-  while (*s) {
-    if (*s != '[') {
-      *d++ = *s++;
-    } else if (s[2] == ']') {
-      *d++ = s[1];
-      s += 3;
-    } else {
-      *d++ = *s++;
-      if (*s == '^') *d++ = *s++;
-      while (*s != ']') {
-        char * min = s;
-        for (char * i = s + 1; *i != ']'; ++i) {
-          if ((byte)*i < (byte)*min) min = i;}
-        char c = *s;
-        *d++ = *min;
-        *min = c;
-        ++s;
-      }
-      *d++ = *s++;
-    }
-  }
-  *d = '\0';
-}
-
 // takes aff file condition string and creates the
 // conds array - please see the appendix at the end of the
 // file affentry.cxx which describes what is going on here
@@ -603,9 +646,8 @@ static void encodeit(CondsLookup & l, ObjStack & buf,
 
   // see if we already have this conds matrix
 
-  normalize_cond_str(cs);
   CondsLookup::iterator itr = l.find(cs);
-  if (itr != l.end()) {
+  if (!(itr == l.end())) {
     ptr->conds = *itr;
     return;
   }
@@ -719,7 +761,7 @@ bool AffixMgr::prefix_check (const LookupInfo & linf, ParmString word,
   // first handle the special case of 0 length prefixes
   PfxEntry * pe = pStart[0];
   while (pe) {
-    if (pe->check(linf,word,ci,gi)) return true;
+    if (pe->check(linf,this,word,ci,gi)) return true;
     pe = pe->next;
   }
   
@@ -729,7 +771,7 @@ bool AffixMgr::prefix_check (const LookupInfo & linf, ParmString word,
 
   while (pptr) {
     if (isSubset(pptr->key(),word)) {
-      if (pptr->check(linf,word,ci,gi)) return true;
+      if (pptr->check(linf,this,word,ci,gi)) return true;
       pptr = pptr->next_eq;
     } else {
       pptr = pptr->next_ne;
@@ -1005,7 +1047,8 @@ SimpleString PfxEntry::add(SimpleString word, ObjStack & buf) const
 }
 
 // check if this prefix entry matches 
-bool PfxEntry::check(const LookupInfo & linf, ParmString word,
+bool PfxEntry::check(const LookupInfo & linf, const AffixMgr * pmyMgr,
+                     ParmString word,
                      CheckInfo & ci, GuessInfo * gi) const
 {
   unsigned int		cond;	// condition number being examined
